# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google/cloud/appengine_v1/proto/application.proto

from cloudsdk.google.protobuf import descriptor as _descriptor
from cloudsdk.google.protobuf import message as _message
from cloudsdk.google.protobuf import reflection as _reflection
from cloudsdk.google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from cloudsdk.google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='google/cloud/appengine_v1/proto/application.proto',
  package='google.appengine.v1',
  syntax='proto3',
  serialized_options=b'\n\027com.google.appengine.v1B\020ApplicationProtoP\001Z<google.golang.org/genproto/googleapis/appengine/v1;appengine',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n1google/cloud/appengine_v1/proto/application.proto\x12\x13google.appengine.v1\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/api/annotations.proto\"\xf0\x07\n\x0b\x41pplication\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\n\n\x02id\x18\x02 \x01(\t\x12<\n\x0e\x64ispatch_rules\x18\x03 \x03(\x0b\x32$.google.appengine.v1.UrlDispatchRule\x12\x13\n\x0b\x61uth_domain\x18\x06 \x01(\t\x12\x13\n\x0blocation_id\x18\x07 \x01(\t\x12\x13\n\x0b\x63ode_bucket\x18\x08 \x01(\t\x12<\n\x19\x64\x65\x66\x61ult_cookie_expiration\x18\t \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x46\n\x0eserving_status\x18\n \x01(\x0e\x32..google.appengine.v1.Application.ServingStatus\x12\x18\n\x10\x64\x65\x66\x61ult_hostname\x18\x0b \x01(\t\x12\x16\n\x0e\x64\x65\x66\x61ult_bucket\x18\x0c \x01(\t\x12@\n\x03iap\x18\x0e \x01(\x0b\x32\x33.google.appengine.v1.Application.IdentityAwareProxy\x12\x12\n\ngcr_domain\x18\x10 \x01(\t\x12\x44\n\rdatabase_type\x18\x11 \x01(\x0e\x32-.google.appengine.v1.Application.DatabaseType\x12J\n\x10\x66\x65\x61ture_settings\x18\x12 \x01(\x0b\x32\x30.google.appengine.v1.Application.FeatureSettings\x1a\x82\x01\n\x12IdentityAwareProxy\x12\x0f\n\x07\x65nabled\x18\x01 \x01(\x08\x12\x18\n\x10oauth2_client_id\x18\x02 \x01(\t\x12\x1c\n\x14oauth2_client_secret\x18\x03 \x01(\t\x12#\n\x1boauth2_client_secret_sha256\x18\x04 \x01(\t\x1aR\n\x0f\x46\x65\x61tureSettings\x12\x1b\n\x13split_health_checks\x18\x01 \x01(\x08\x12\"\n\x1ause_container_optimized_os\x18\x02 \x01(\x08\"U\n\rServingStatus\x12\x0f\n\x0bUNSPECIFIED\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x12\x11\n\rUSER_DISABLED\x10\x02\x12\x13\n\x0fSYSTEM_DISABLED\x10\x03\"z\n\x0c\x44\x61tabaseType\x12\x1d\n\x19\x44\x41TABASE_TYPE_UNSPECIFIED\x10\x00\x12\x13\n\x0f\x43LOUD_DATASTORE\x10\x01\x12\x13\n\x0f\x43LOUD_FIRESTORE\x10\x02\x12!\n\x1d\x43LOUD_DATASTORE_COMPATIBILITY\x10\x03\"@\n\x0fUrlDispatchRule\x12\x0e\n\x06\x64omain\x18\x01 \x01(\t\x12\x0c\n\x04path\x18\x02 \x01(\t\x12\x0f\n\x07service\x18\x03 \x01(\tBk\n\x17\x63om.google.appengine.v1B\x10\x41pplicationProtoP\x01Z<google.golang.org/genproto/googleapis/appengine/v1;appengineb\x06proto3'
  ,
  dependencies=[google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_api_dot_annotations__pb2.DESCRIPTOR,])



_APPLICATION_SERVINGSTATUS = _descriptor.EnumDescriptor(
  name='ServingStatus',
  full_name='google.appengine.v1.Application.ServingStatus',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SERVING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='USER_DISABLED', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='SYSTEM_DISABLED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=936,
  serialized_end=1021,
)
_sym_db.RegisterEnumDescriptor(_APPLICATION_SERVINGSTATUS)

_APPLICATION_DATABASETYPE = _descriptor.EnumDescriptor(
  name='DatabaseType',
  full_name='google.appengine.v1.Application.DatabaseType',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='DATABASE_TYPE_UNSPECIFIED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CLOUD_DATASTORE', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CLOUD_FIRESTORE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='CLOUD_DATASTORE_COMPATIBILITY', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1023,
  serialized_end=1145,
)
_sym_db.RegisterEnumDescriptor(_APPLICATION_DATABASETYPE)


_APPLICATION_IDENTITYAWAREPROXY = _descriptor.Descriptor(
  name='IdentityAwareProxy',
  full_name='google.appengine.v1.Application.IdentityAwareProxy',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='enabled', full_name='google.appengine.v1.Application.IdentityAwareProxy.enabled', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='oauth2_client_id', full_name='google.appengine.v1.Application.IdentityAwareProxy.oauth2_client_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='oauth2_client_secret', full_name='google.appengine.v1.Application.IdentityAwareProxy.oauth2_client_secret', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='oauth2_client_secret_sha256', full_name='google.appengine.v1.Application.IdentityAwareProxy.oauth2_client_secret_sha256', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=720,
  serialized_end=850,
)

_APPLICATION_FEATURESETTINGS = _descriptor.Descriptor(
  name='FeatureSettings',
  full_name='google.appengine.v1.Application.FeatureSettings',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='split_health_checks', full_name='google.appengine.v1.Application.FeatureSettings.split_health_checks', index=0,
      number=1, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='use_container_optimized_os', full_name='google.appengine.v1.Application.FeatureSettings.use_container_optimized_os', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=852,
  serialized_end=934,
)

_APPLICATION = _descriptor.Descriptor(
  name='Application',
  full_name='google.appengine.v1.Application',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='google.appengine.v1.Application.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='id', full_name='google.appengine.v1.Application.id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='dispatch_rules', full_name='google.appengine.v1.Application.dispatch_rules', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='auth_domain', full_name='google.appengine.v1.Application.auth_domain', index=3,
      number=6, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='location_id', full_name='google.appengine.v1.Application.location_id', index=4,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='code_bucket', full_name='google.appengine.v1.Application.code_bucket', index=5,
      number=8, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_cookie_expiration', full_name='google.appengine.v1.Application.default_cookie_expiration', index=6,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='serving_status', full_name='google.appengine.v1.Application.serving_status', index=7,
      number=10, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_hostname', full_name='google.appengine.v1.Application.default_hostname', index=8,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='default_bucket', full_name='google.appengine.v1.Application.default_bucket', index=9,
      number=12, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='iap', full_name='google.appengine.v1.Application.iap', index=10,
      number=14, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='gcr_domain', full_name='google.appengine.v1.Application.gcr_domain', index=11,
      number=16, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='database_type', full_name='google.appengine.v1.Application.database_type', index=12,
      number=17, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='feature_settings', full_name='google.appengine.v1.Application.feature_settings', index=13,
      number=18, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[_APPLICATION_IDENTITYAWAREPROXY, _APPLICATION_FEATURESETTINGS, ],
  enum_types=[
    _APPLICATION_SERVINGSTATUS,
    _APPLICATION_DATABASETYPE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=137,
  serialized_end=1145,
)


_URLDISPATCHRULE = _descriptor.Descriptor(
  name='UrlDispatchRule',
  full_name='google.appengine.v1.UrlDispatchRule',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='domain', full_name='google.appengine.v1.UrlDispatchRule.domain', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='path', full_name='google.appengine.v1.UrlDispatchRule.path', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='service', full_name='google.appengine.v1.UrlDispatchRule.service', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1147,
  serialized_end=1211,
)

_APPLICATION_IDENTITYAWAREPROXY.containing_type = _APPLICATION
_APPLICATION_FEATURESETTINGS.containing_type = _APPLICATION
_APPLICATION.fields_by_name['dispatch_rules'].message_type = _URLDISPATCHRULE
_APPLICATION.fields_by_name['default_cookie_expiration'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_APPLICATION.fields_by_name['serving_status'].enum_type = _APPLICATION_SERVINGSTATUS
_APPLICATION.fields_by_name['iap'].message_type = _APPLICATION_IDENTITYAWAREPROXY
_APPLICATION.fields_by_name['database_type'].enum_type = _APPLICATION_DATABASETYPE
_APPLICATION.fields_by_name['feature_settings'].message_type = _APPLICATION_FEATURESETTINGS
_APPLICATION_SERVINGSTATUS.containing_type = _APPLICATION
_APPLICATION_DATABASETYPE.containing_type = _APPLICATION
DESCRIPTOR.message_types_by_name['Application'] = _APPLICATION
DESCRIPTOR.message_types_by_name['UrlDispatchRule'] = _URLDISPATCHRULE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Application = _reflection.GeneratedProtocolMessageType('Application', (_message.Message,), {

  'IdentityAwareProxy' : _reflection.GeneratedProtocolMessageType('IdentityAwareProxy', (_message.Message,), {
    'DESCRIPTOR' : _APPLICATION_IDENTITYAWAREPROXY,
    '__module__' : 'google.cloud.appengine_v1.proto.application_pb2'
    ,
    '__doc__': """Identity-Aware Proxy
    
    Attributes:
        enabled:
            Whether the serving infrastructure will authenticate and
            authorize all incoming requests.  If true, the
            ``oauth2_client_id`` and ``oauth2_client_secret`` fields must
            be non-empty.
        oauth2_client_id:
            OAuth2 client ID to use for the authentication flow.
        oauth2_client_secret:
            OAuth2 client secret to use for the authentication flow.  For
            security reasons, this value cannot be retrieved via the API.
            Instead, the SHA-256 hash of the value is returned in the
            ``oauth2_client_secret_sha256`` field.  @InputOnly
        oauth2_client_secret_sha256:
            Hex-encoded SHA-256 hash of the client secret.  @OutputOnly
    """,
    # @@protoc_insertion_point(class_scope:google.appengine.v1.Application.IdentityAwareProxy)
    })
  ,

  'FeatureSettings' : _reflection.GeneratedProtocolMessageType('FeatureSettings', (_message.Message,), {
    'DESCRIPTOR' : _APPLICATION_FEATURESETTINGS,
    '__module__' : 'google.cloud.appengine_v1.proto.application_pb2'
    ,
    '__doc__': """The feature specific settings to be used in the application. These
    define behaviors that are user configurable.
    
    Attributes:
        split_health_checks:
            Boolean value indicating if split health checks should be used
            instead of the legacy health checks. At an app.yaml level,
            this means defaulting to ‘readiness_check’ and
            ‘liveness_check’ values instead of ‘health_check’ ones. Once
            the legacy ‘health_check’ behavior is deprecated, and this
            value is always true, this setting can be removed.
        use_container_optimized_os:
            If true, use `Container-Optimized OS
            <https://cloud.google.com/container-optimized-os/>`__ base
            image for VMs, rather than a base Debian image.
    """,
    # @@protoc_insertion_point(class_scope:google.appengine.v1.Application.FeatureSettings)
    })
  ,
  'DESCRIPTOR' : _APPLICATION,
  '__module__' : 'google.cloud.appengine_v1.proto.application_pb2'
  ,
  '__doc__': """An Application resource contains the top-level configuration of an App
  Engine application.
  
  Attributes:
      name:
          Full path to the Application resource in the API. Example:
          ``apps/myapp``.  @OutputOnly
      id:
          Identifier of the Application resource. This identifier is
          equivalent to the project ID of the Google Cloud Platform
          project where you want to deploy your application. Example:
          ``myapp``.
      dispatch_rules:
          HTTP path dispatch rules for requests to the application that
          do not explicitly target a service or version. Rules are
          order-dependent. Up to 20 dispatch rules can be supported.
      auth_domain:
          Google Apps authentication domain that controls which users
          can access this application.  Defaults to open access for any
          Google Account.
      location_id:
          Location from which this application runs. Application
          instances run out of the data centers in the specified
          location, which is also where all of the application’s end
          user content is stored.  Defaults to ``us-central``.  View the
          list of `supported locations
          <https://cloud.google.com/appengine/docs/locations>`__.
      code_bucket:
          Google Cloud Storage bucket that can be used for storing files
          associated with this application. This bucket is associated
          with the application and can be used by the gcloud deployment
          commands.  @OutputOnly
      default_cookie_expiration:
          Cookie expiration policy for this application.
      serving_status:
          Serving status of this application.
      default_hostname:
          Hostname used to reach this application, as resolved by App
          Engine.  @OutputOnly
      default_bucket:
          Google Cloud Storage bucket that can be used by this
          application to store content.  @OutputOnly
      gcr_domain:
          The Google Container Registry domain used for storing managed
          build docker images for this application.
      database_type:
          The type of the Cloud Firestore or Cloud Datastore database
          associated with this application.
      feature_settings:
          The feature specific settings to be used in the application.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.Application)
  })
_sym_db.RegisterMessage(Application)
_sym_db.RegisterMessage(Application.IdentityAwareProxy)
_sym_db.RegisterMessage(Application.FeatureSettings)

UrlDispatchRule = _reflection.GeneratedProtocolMessageType('UrlDispatchRule', (_message.Message,), {
  'DESCRIPTOR' : _URLDISPATCHRULE,
  '__module__' : 'google.cloud.appengine_v1.proto.application_pb2'
  ,
  '__doc__': """Rules to match an HTTP request and dispatch that request to a service.
  
  Attributes:
      domain:
          Domain name to match against. The wildcard “``*``” is
          supported if specified before a period: “``*.``”.  Defaults to
          matching all domains: “``*``”.
      path:
          Pathname within the host. Must start with a “``/``”. A single
          “``*``” can be included at the end of the path.  The sum of
          the lengths of the domain and path may not exceed 100
          characters.
      service:
          Resource ID of a service in this application that should serve
          the matched request. The service must already exist. Example:
          ``default``.
  """,
  # @@protoc_insertion_point(class_scope:google.appengine.v1.UrlDispatchRule)
  })
_sym_db.RegisterMessage(UrlDispatchRule)


DESCRIPTOR._options = None
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Workflow to set up gcloud environment."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import os

from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions as c_exc
from googlecloudsdk.calliope import usage_text
from googlecloudsdk.command_lib import init_util
from googlecloudsdk.core import config
from googlecloudsdk.core import execution_utils
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import yaml
from googlecloudsdk.core.configurations import named_configs
from googlecloudsdk.core.console import console_io
from googlecloudsdk.core.credentials import store as c_store
from googlecloudsdk.core.diagnostics import network_diagnostics
from googlecloudsdk.core.resource import resource_projector
from googlecloudsdk.core.util import files
from googlecloudsdk.core.util import platforms

import six


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.BETA,
                    base.ReleaseTrack.GA)
class Init(base.Command):
  """Initialize or reinitialize gcloud.

  {command} launches an interactive Getting Started workflow for the gcloud
  command-line tool.
  It performs the following setup steps:

  - Authorizes gcloud and other SDK tools to access Google Cloud using
    your user account credentials, or from an account of your choosing whose
    credentials are already available.
  - Sets up a new or existing configuration.
  - Sets properties in that configuration, including the current project and
    optionally, the default Google Compute Engine region and zone you'd like to
    use.

  {command} can be used for initial setup of gcloud and to create new or
  reinitialize gcloud configurations. More information about configurations can
  be found by running `gcloud topic configurations`.

  Properties set by {command} are local and persistent, and are not affected by
  remote changes to the project. For example, the default Compute Engine zone in
  your configuration remains stable, even if you or another user changes the
  project-level default zone in the Cloud Platform Console.

  To sync the configuration, re-run `{command}`.

  ## EXAMPLES

  To launch an interactive Getting Started workflow, run:

    $ {command}

  To launch an interactive Getting Started workflow without diagnostics, run:

    $ {command} --skip-diagnostics

  """

  category = base.SDK_TOOLS_CATEGORY

  @staticmethod
  def Args(parser):
    parser.add_argument(
        'obsolete_project_arg',
        nargs='?',
        hidden=True,
        help='THIS ARGUMENT NEEDS HELP TEXT.')
    parser.add_argument(
        '--console-only',
        '--no-launch-browser',
        help=(
            'Prevent the command from launching a browser for '
            'authorization. Use this flag if you are on a machine that '
            'does not have a browser and you cannot install the '
            'gcloud CLI on another machine with a browser.'
        ),
        action='store_true',
    )
    parser.add_argument(
        '--no-browser',
        help=(
            'Prevent the command from launching a browser for '
            'authorization. Use this flag if you are on a machine that '
            'does not have a browser but you can install the '
            'gcloud CLI on another machine with a browser.'
        ),
        action='store_true',
    )
    parser.add_argument(
        '--skip-diagnostics',
        action='store_true',
        help='Do not run diagnostics.')

  def Run(self, args):
    """Allows user to select configuration, and initialize it."""
    if args.obsolete_project_arg:
      raise c_exc.InvalidArgumentException(
          args.obsolete_project_arg,
          '`gcloud init` has changed and no longer takes a PROJECT argument. '
          'Please use `gcloud source repos clone` to clone this '
          'project\'s source repositories.')

    log.status.write('Welcome! This command will take you through '
                     'the configuration of gcloud.\n\n')

    if properties.VALUES.core.disable_prompts.GetBool():
      raise c_exc.InvalidArgumentException(
          'disable_prompts/--quiet',
          'gcloud init command cannot run with disabled prompts.')

    configuration_name = self._PickConfiguration()
    if not configuration_name:
      return
    log.status.write('Your current configuration has been set to: [{0}]\n\n'
                     .format(configuration_name))

    if not args.skip_diagnostics:
      log.status.write('You can skip diagnostics next time by using the '
                       'following flag:\n')
      log.status.write('  gcloud init --skip-diagnostics\n\n')
      network_passed = network_diagnostics.NetworkDiagnostic().RunChecks()
      if not network_passed:
        if not console_io.PromptContinue(
            message='Network errors detected.',
            prompt_string='Would you like to continue anyway',
            default=False):
          log.status.write('You can re-run diagnostics with the following '
                           'command:\n')
          log.status.write('  gcloud info --run-diagnostics\n\n')
          return

    # User project quota is now the global default, but this command calls
    # legacy APIs where it should be disabled.
    with base.WithLegacyQuota():
      if not self._PickAccount(
          args.console_only, args.no_browser, preselected=args.account):
        return

      if not self._PickProject(preselected=args.project):
        return

      self._PickDefaultRegionAndZone()

      self._CreateBotoConfig()

      self._Summarize(configuration_name)

  def _PickAccount(self, console_only, no_browser, preselected=None):
    """Checks if current credentials are valid, if not runs auth login.

    Args:
      console_only: bool, True if the auth flow shouldn't use the browser
      no_browser: bool, True if the auth flow shouldn't use the browser and
        should ask another gcloud installation to help with the browser flow.
      preselected: str, disable prompts and use this value if not None

    Returns:
      bool, True if valid credentials are setup.
    """

    new_credentials = False
    accounts = c_store.AvailableAccounts()
    if accounts:
      # There is at least one credentialed account.
      if preselected:
        # Try to use the preselected account. Fail if its not credentialed.
        account = preselected
        if account not in accounts:
          log.status.write('\n[{0}] is not one of your credentialed accounts '
                           '[{1}].\n'.format(account, ','.join(accounts)))
          return False
        # Fall through to the set the account property.
      else:
        # Prompt for the account to use.
        idx = console_io.PromptChoice(
            accounts + ['Log in with a new account'],
            message='Choose the account you would like to use to perform '
                    'operations for this configuration:',
            prompt_string=None)
        if idx is None:
          return False
        if idx < len(accounts):
          account = accounts[idx]
        else:
          new_credentials = True
    elif preselected:
      # Preselected account specified but there are no credentialed accounts.
      log.status.write('\n[{0}] is not a credentialed account.\n'.format(
          preselected))
      return False
    else:
      # Must log in with new credentials.
      answer = console_io.PromptContinue(
          prompt_string='You must log in to continue. Would you like to log in')
      if not answer:
        return False
      new_credentials = True
    if new_credentials:
      # Call `gcloud auth login` to get new credentials.
      # `gcloud auth login` may have user interaction, do not suppress it.
      if console_only:
        browser_args = ['--no-launch-browser']
      elif no_browser:
        browser_args = ['--no-browser']
      else:
        browser_args = []
      if not self._RunCmd(['auth', 'login'],
                          ['--force', '--brief'] + browser_args,
                          disable_user_output=False):
        return False
      # `gcloud auth login` already did `gcloud config set account`.
    else:
      # Set the config account to the already credentialed account.
      properties.PersistProperty(properties.VALUES.core.account, account)

    log.status.write('You are logged in as: [{0}].\n\n'
                     .format(properties.VALUES.core.account.Get()))
    return True

  def _PickConfiguration(self):
    """Allows user to re-initialize, create or pick new configuration.

    Returns:
      Configuration name or None.
    """
    configs = named_configs.ConfigurationStore.AllConfigs()
    active_config = named_configs.ConfigurationStore.ActiveConfig()

    if not configs or active_config.name not in configs:
      # Listing the configs will automatically create the default config.  The
      # only way configs could be empty here is if there are no configurations
      # and the --configuration flag or env var is set to something that does
      # not exist.  If configs has items, but the active config is not in there,
      # that similarly means that hey are using the flag or the env var and that
      # config does not exist.  In either case, just create it and go with that
      # one as the one that as they have already selected it.
      named_configs.ConfigurationStore.CreateConfig(active_config.name)
      # Need to active it in the file, not just the environment.
      active_config.Activate()
      return active_config.name

    # If there is a only 1 config, it is the default, and there are no
    # properties set, assume it was auto created and that it should be
    # initialized as if it didn't exist.
    if len(configs) == 1:
      default_config = configs.get(named_configs.DEFAULT_CONFIG_NAME, None)
      if default_config and not default_config.GetProperties():
        default_config.Activate()
        return default_config.name

    choices = []
    log.status.write('Settings from your current configuration [{0}] are:\n'
                     .format(active_config.name))
    log.status.flush()
    log.status.write(yaml.dump(properties.VALUES.AllValues()))
    log.out.flush()
    log.status.write('\n')
    log.status.flush()
    choices.append(
        'Re-initialize this configuration [{0}] with new settings '.format(
            active_config.name))
    choices.append('Create a new configuration')
    config_choices = [name for name, c in sorted(six.iteritems(configs))
                      if not c.is_active]
    choices.extend('Switch to and re-initialize '
                   'existing configuration: [{0}]'.format(name)
                   for name in config_choices)
    idx = console_io.PromptChoice(choices, message='Pick configuration to use:')
    if idx is None:
      return None
    if idx == 0:  # If reinitialize was selected.
      self._CleanCurrentConfiguration()
      return active_config.name
    if idx == 1:  # Second option is to create new configuration.
      return self._CreateConfiguration()
    config_name = config_choices[idx - 2]
    named_configs.ConfigurationStore.ActivateConfig(config_name)
    return config_name

  def _PickProject(self, preselected=None):
    """Allows user to select a project.

    Args:
      preselected: str, use this value if not None

    Returns:
      str, project_id or None if was not selected.
    """
    project_id = init_util.PickProject(preselected=preselected)
    if project_id is not None:
      properties.PersistProperty(properties.VALUES.core.project, project_id)
      log.status.write('Your current project has been set to: [{0}].\n\n'
                       .format(project_id))
    return project_id

  def _PickDefaultRegionAndZone(self):
    """Pulls metadata properties for region and zone and sets them in gcloud."""
    try:
      # Use --quiet flag to skip the enable api prompt.
      project_info = self._RunCmd(['compute', 'project-info', 'describe'],
                                  params=['--quiet'])
    except Exception:  # pylint:disable=broad-except
      log.status.write("""\
Not setting default zone/region (this feature makes it easier to use
[gcloud compute] by setting an appropriate default value for the
--zone and --region flag).
See https://cloud.google.com/compute/docs/gcloud-compute section on how to set
default compute region and zone manually. If you would like [gcloud init] to be
able to do this for you the next time you run it, make sure the
Compute Engine API is enabled for your project on the
https://console.developers.google.com/apis page.

""")
      return None

    default_zone = None
    default_region = None
    if project_info is not None:
      project_info = resource_projector.MakeSerializable(project_info)
      metadata = project_info.get('commonInstanceMetadata', {})
      for item in metadata.get('items', []):
        if item['key'] == 'google-compute-default-zone':
          default_zone = item['value']
        elif item['key'] == 'google-compute-default-region':
          default_region = item['value']

    # We could not determine zone automatically. Before offering choices for
    # zone and/or region ask user if he/she wants to do this.
    if not default_zone:
      answer = console_io.PromptContinue(
          prompt_string=('Do you want to configure a default Compute '
                         'Region and Zone?'))
      if not answer:
        return

    # Same logic applies to region and zone properties.
    def SetProperty(name, default_value, list_command):
      """Set named compute property to default_value or get via list command."""
      if not default_value:
        values = self._RunCmd(list_command)
        if values is None:
          return
        values = list(values)
        message = (
            'Which Google Compute Engine {0} would you like to use as project '
            'default?\n'
            'If you do not specify a {0} via a command line flag while working '
            'with Compute Engine resources, the default is assumed.').format(
                name)
        idx = console_io.PromptChoice(
            [value['name'] for value in values]
            + ['Do not set default {0}'.format(name)],
            message=message, prompt_string=None, allow_freeform=True,
            freeform_suggester=usage_text.TextChoiceSuggester())
        if idx is None or idx == len(values):
          return
        default_value = values[idx]
      properties.PersistProperty(properties.VALUES.compute.Property(name),
                                 default_value['name'])
      log.status.write('Your project default Compute Engine {0} has been set '
                       'to [{1}].\nYou can change it by running '
                       '[gcloud config set compute/{0} NAME].\n\n'
                       .format(name, default_value['name']))
      return default_value

    if default_zone:
      default_zone = self._RunCmd(['compute', 'zones', 'describe'],
                                  [default_zone])
    zone = SetProperty('zone', default_zone, ['compute', 'zones', 'list'])
    if zone and not default_region:
      default_region = zone['region']
    if default_region:
      default_region = self._RunCmd(['compute', 'regions', 'describe'],
                                    [default_region])
    SetProperty('region', default_region, ['compute', 'regions', 'list'])

  def _Summarize(self, configuration_name):
    log.status.Print('Your Google Cloud SDK is configured and ready to use!\n')

    log.status.Print(
        '* Commands that require authentication will use {0} by default'
        .format(properties.VALUES.core.account.Get()))
    project = properties.VALUES.core.project.Get()
    if project:
      log.status.Print(
          '* Commands will reference project `{0}` by default'
          .format(project))
    region = properties.VALUES.compute.region.Get()
    if region:
      log.status.Print(
          '* Compute Engine commands will use region `{0}` by default'
          .format(region))
    zone = properties.VALUES.compute.zone.Get()
    if zone:
      log.status.Print(
          '* Compute Engine commands will use zone `{0}` by default\n'
          .format(zone))

    log.status.Print(
        'Run `gcloud help config` to learn how to change individual settings\n')

    log.status.Print(
        'This gcloud configuration is called [{config}]. You can create '
        'additional configurations if you work with multiple accounts and/or '
        'projects.'.format(config=configuration_name))
    log.status.Print('Run `gcloud topic configurations` to learn more.\n')

    log.status.Print('Some things to try next:\n')

    log.status.Print(
        '* Run `gcloud --help` to see the Cloud Platform services you can '
        'interact with. And run `gcloud help COMMAND` to get help on any '
        'gcloud command.')
    log.status.Print(
        '* Run `gcloud topic --help` to learn about advanced features of the '
        'SDK like arg files and output formatting')

    log.status.Print(
        '* Run `gcloud cheat-sheet` to see a roster of go-to `gcloud` '
        'commands.')

  def _CreateConfiguration(self):
    configuration_name = console_io.PromptResponse(
        'Enter configuration name. Names start with a lower case letter and '
        'contain only lower case letters a-z, digits 0-9, and hyphens \'-\':  ')
    configuration_name = configuration_name.strip()
    named_configs.ConfigurationStore.CreateConfig(configuration_name)
    named_configs.ConfigurationStore.ActivateConfig(configuration_name)
    named_configs.ActivePropertiesFile.Invalidate()
    return configuration_name

  def _CreateBotoConfig(self):
    gsutil_path = _FindGsutil()
    if not gsutil_path:
      log.debug('Unable to find [gsutil]. Not configuring default .boto '
                'file')
      return

    boto_path = files.ExpandHomeDir(os.path.join('~', '.boto'))
    if os.path.exists(boto_path):
      log.debug('Not configuring default .boto file. File already '
                'exists at [{boto_path}].'.format(boto_path=boto_path))
      return

    # 'gsutil config -n' creates a default .boto file that the user can read and
    # modify.
    command_args = ['config', '-n', '-o', boto_path]
    if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
      gsutil_args = execution_utils.ArgsForCMDTool(gsutil_path,
                                                   *command_args)
    else:
      gsutil_args = execution_utils.ArgsForExecutableTool(gsutil_path,
                                                          *command_args)

    return_code = execution_utils.Exec(gsutil_args, no_exit=True,
                                       out_func=log.file_only_logger.debug,
                                       err_func=log.file_only_logger.debug)
    if return_code == 0:
      log.status.write("""\
Created a default .boto configuration file at [{boto_path}]. See this file and
[https://cloud.google.com/storage/docs/gsutil/commands/config] for more
information about configuring Google Cloud Storage.
""".format(boto_path=boto_path))

    else:
      log.status.write('Error creating a default .boto configuration file. '
                       'Please run [gsutil config -n] if you would like to '
                       'create this file.\n')

  def _CleanCurrentConfiguration(self):
    properties.PersistProperty(properties.VALUES.core.account, None)
    properties.PersistProperty(properties.VALUES.core.project, None)
    properties.PersistProperty(properties.VALUES.compute.region, None)
    properties.PersistProperty(properties.VALUES.compute.zone, None)
    named_configs.ActivePropertiesFile.Invalidate()

  def _RunCmd(self, cmd, params=None, disable_user_output=True):
    if not self._cli_power_users_only.IsValidCommand(cmd):
      log.info('Command %s does not exist.', cmd)
      return None
    if params is None:
      params = []
    args = cmd + params
    log.info('Executing: [gcloud %s]', ' '.join(args))
    try:
      # Disable output from individual commands, so that we get
      # command run results, and don't clutter output of init.
      if disable_user_output:
        args.append('--no-user-output-enabled')

      if (properties.VALUES.core.verbosity.Get() is None and
          disable_user_output):
        # Unless user explicitly set verbosity, suppress from subcommands.
        args.append('--verbosity=none')

      if properties.VALUES.core.log_http.GetBool():
        args.append('--log-http')

      return resource_projector.MakeSerializable(
          self.ExecuteCommandDoNotUse(args))

    except SystemExit as exc:
      log.info('[%s] has failed\n', ' '.join(cmd + params))
      raise c_exc.FailedSubCommand(cmd + params, exc.code)
    except BaseException:
      log.info('Failed to run [%s]\n', ' '.join(cmd + params))
      raise


def _FindGsutil():
  """Finds the bundled gsutil wrapper.

  Returns:
    The path to gsutil.
  """
  sdk_bin_path = config.Paths().sdk_bin_path
  if not sdk_bin_path:
    return

  if platforms.OperatingSystem.Current() == platforms.OperatingSystem.WINDOWS:
    gsutil = 'gsutil.cmd'
  else:
    gsutil = 'gsutil'
  return os.path.join(sdk_bin_path, gsutil)



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The unregister-cluster command for removing clusters from the fleet."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

import textwrap

from apitools.base.py import exceptions as apitools_exceptions
from googlecloudsdk.api_lib.container.fleet import util
from googlecloudsdk.calliope import base as calliope_base
from googlecloudsdk.command_lib.container.fleet import agent_util
from googlecloudsdk.command_lib.container.fleet import api_util
from googlecloudsdk.command_lib.container.fleet import exclusivity_util
from googlecloudsdk.command_lib.container.fleet import kube_util
from googlecloudsdk.command_lib.container.fleet import resources
from googlecloudsdk.command_lib.container.fleet import util as hub_util
from googlecloudsdk.command_lib.container.fleet.memberships import gke_util
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.core import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


class Unregister(calliope_base.DeleteCommand):
  r"""Unregister a cluster from a fleet.

  This command unregisters a cluster with the fleet by:

    1. Deleting the Fleet Membership resource for this cluster (a.k.a
       `{parent_command} delete`).
    2. Removing the corresponding in-cluster Kubernetes Resources that make the
       cluster exclusive to one fleet (a.k.a `kubectl delete memberships
       membership`).
    3. [Optional for GKE cluster] Uninstalling the Connect agent from this
       cluster (a.k.a `kubectl delete on the gke-connect namespace`).

  The unregister command makes additional internal checks to ensure that all
  three steps can be safely done to properly clean-up in-Fleet and in-cluster
  resources.

  To register a GKE cluster use `--gke-cluster` or `--gke-uri` flag (no
  `--kubeconfig` flag is required).

  To register a non-GKE cluster use `--context` flag (with an optional
  -`-kubeconfig`s flag).

  To only delete the Fleet Membership resource, consider using the command:
  `{parent_command} delete`. This command is intended to delete stale Fleet
  Membership resources as doing so on a fully registered cluster will skip some
  of the steps above and orphan in-cluster resources and agent connections to
  Google.

  ## EXAMPLES

    Unregister a non-GKE cluster referenced from a specific kubeconfig file:

      $ {command} my-membership \
        --context=my-cluster-context \
        --kubeconfig=/home/user/custom_kubeconfig

    Unregister a non-GKE cluster referenced from the default kubeconfig file:

      $ {command} my-membership --context=my-cluster-context

    Unregister a GKE cluster referenced from a GKE URI:

      $ {command} my-membership \
        --gke-uri=my-cluster-gke-uri

    Unregister a GKE cluster referenced from a GKE Cluster location and name:

      $ {command} my-membership \
        --gke-cluster=my-cluster-region-or-zone/my-cluster

    Unregister a GKE cluster and uninstall Connect agent:

      $ {command} my-membership \
        --gke-cluster=my-cluster-region-or-zone/my-cluster
        --uninstall-connect-agent

  """

  @classmethod
  def Args(cls, parser):
    resources.AddMembershipResourceArg(
        parser,
        membership_help=textwrap.dedent("""\
          The membership name that you choose to uniquely represent the cluster
          being registered in the fleet.
        """),
        location_help=textwrap.dedent("""\
          The location of the membership resource, e.g. `us-central1`.
          If not specified, defaults to `global`.
        """),
        membership_required=True,
        positional=True)
    parser.add_argument(
        '--uninstall-connect-agent',
        action='store_true',
        help=textwrap.dedent("""\
          If set to True for a GKE cluster, Connect agent will be uninstalled
          from the cluster. No-op for Non-GKE clusters, where Connect agent will
          always be uninstalled.
          """),
        default=False,
    )
    hub_util.AddClusterConnectionCommonArgs(parser)

  def Run(self, args):
    project = arg_utils.GetFromNamespace(args, '--project', use_defaults=True)
    gke_cluster_resource_link, gke_cluster_uri = (
        gke_util.GetGKEClusterResoureLinkAndURI(
            gke_uri=args.GetValue('gke_uri'),
            gke_cluster=args.GetValue('gke_cluster'),
        )
    )
    location = 'global'
    membership_id = args.MEMBERSHIP_NAME
    if resources.MembershipLocationSpecified(args):
      membership = resources.MembershipResourceName(args)
      membership_id = util.MembershipShortname(membership)
      location = util.MembershipLocation(membership)
    # If this is a gke cluster and location is ambiguous, then we'll check for
    # the existence of a regional membership. Non-gke clusters can only be
    # registered as 'global' memberships right now.
    elif gke_cluster_resource_link:
      cluster_location = hub_util.LocationFromGKEArgs(args)
      regional_name = 'projects/{}/locations/{}/memberships/{}'.format(
          project, cluster_location, membership_id
      )
      try:
        regional_obj = api_util.GetMembership(
            regional_name, self.ReleaseTrack()
        )
        if (
            regional_obj.endpoint
            and regional_obj.endpoint.gkeCluster.resourceLink
            == gke_cluster_resource_link
        ):
          location = cluster_location
      except apitools_exceptions.HttpError:
        pass

    # Unregister GKE cluster with simple Add-to-Hub API call. Connect agent will
    # not be uninstalled. And Kubernetes Client is not needed.
    if gke_cluster_resource_link and not args.GetValue(
        'uninstall_connect_agent'):
      return self._UnregisterGKE(gke_cluster_resource_link, gke_cluster_uri,
                                 project, location, membership_id, args)

    # Unregister non-GKE, or GKE with --uninstall-connect-agent.
    # It will require a kube client.
    kube_client = kube_util.KubernetesClient(
        gke_uri=getattr(args, 'gke_uri', None),
        gke_cluster=getattr(args, 'gke_cluster', None),
        kubeconfig=getattr(args, 'kubeconfig', None),
        context=getattr(args, 'context', None),
        public_issuer_url=getattr(args, 'public_issuer_url', None),
        enable_workload_identity=getattr(args, 'enable_workload_identity',
                                         False),
    )
    kube_client.CheckClusterAdminPermissions()
    kube_util.ValidateClusterIdentifierFlags(kube_client, args)

    # Delete membership from Fleet API.
    try:
      name = 'projects/{}/locations/{}/memberships/{}'.format(
          project, location, membership_id)
      obj = api_util.GetMembership(name, self.ReleaseTrack())
      if not obj.externalId:
        console_io.PromptContinue(
            'invalid membership {0} does not have '
            'external_id field set. We cannot determine '
            'if registration is requested against a '
            'valid existing Membership. Consult the '
            'documentation on container fleet memberships '
            'update for more information or run gcloud '
            'container fleet memberships delete {0} if you '
            'are sure that this is an invalid or '
            'otherwise stale Membership'.format(membership_id),
            cancel_on_no=True)
      uuid = kube_util.GetClusterUUID(kube_client)
      if obj.externalId != uuid:
        raise exceptions.Error(
            'Membership [{}] is not associated with the cluster you are trying'
            ' to unregister. Please double check the cluster identifier that you'
            ' have supplied.'.format(name))

      api_util.DeleteMembership(name, self.ReleaseTrack())
    except apitools_exceptions.HttpUnauthorizedError as e:
      raise exceptions.Error(
          'You are not authorized to unregister clusters from project [{}]. '
          'Underlying error: {}'.format(project, e))
    except apitools_exceptions.HttpNotFoundError:
      log.status.Print(
          'Membership [{}] for the cluster was not found on the fleet. '
          'It may already have been deleted, or it may never have existed.'
          .format(name))

    # Get namespace for the connect resource label.
    selector = '{}={}'.format(agent_util.CONNECT_RESOURCE_LABEL, project)
    namespaces = kube_client.NamespacesWithLabelSelector(selector)
    if not namespaces:
      log.status.Print('There\'s no namespace for the label [{}]. '
                       'If [gke-connect] is labeled with another project, '
                       'You\'ll have to manually delete the namespace. '
                       'You can find all namespaces by running:\n'
                       '  `kubectl get ns -l {}`'.format(
                           agent_util.CONNECT_RESOURCE_LABEL,
                           agent_util.CONNECT_RESOURCE_LABEL))

    # Delete in-cluster membership resources.
    try:
      parent = api_util.ParentRef(project, location)
      cr_manifest = kube_client.GetMembershipCR()

      res = api_util.ValidateExclusivity(cr_manifest, parent, membership_id,
                                         self.ReleaseTrack())
      if res.status.code:
        console_io.PromptContinue(
            'Error validating cluster\'s exclusivity state with the Fleet under '
            'parent collection [{}]: {}. The cluster you are trying to unregister'
            ' is not associated with the membership [{}]. Continuing will delete'
            ' membership related resources from your cluster, and the cluster'
            ' will lose its association to the Fleet in project [{}] and can be'
            ' registered into a different project. '.format(
                parent, res.status.message, membership_id, project),
            cancel_on_no=True)
      exclusivity_util.DeleteMembershipResources(kube_client)
    except exceptions.Error as e:
      log.status.Print(
          '{} error in deleting in-cluster membership resources. '
          'You can manually delete these membership related '
          'resources from your cluster by running the command:\n'
          '  `kubectl delete memberships membership`.\nBy doing so, '
          'the cluster will lose its association to the Fleet in '
          'project [{}] and can be registered into a different '
          'project. '.format(e, project))

    # Delete the connect agent.
    agent_util.DeleteConnectNamespace(kube_client, args)

  def _UnregisterGKE(self, gke_cluster_resource_link, gke_cluster_uri, project,
                     location, membership_id, args):
    """Register a GKE cluster without installing Connect agent."""
    try:
      name = 'projects/{}/locations/{}/memberships/{}'.format(
          project, location, membership_id)
      obj = api_util.GetMembership(name, self.ReleaseTrack())
      if obj.endpoint.gkeCluster.resourceLink != gke_cluster_resource_link:
        raise exceptions.Error(
            'membership [{0}] is associated with a different GKE cluster link '
            '{1}. You may be unregistering the wrong membership.'.format(
                name, obj.endpoint.gkeCluster.resourceLink))

      api_util.DeleteMembership(name, self.ReleaseTrack())
    except apitools_exceptions.HttpUnauthorizedError as e:
      raise exceptions.Error(
          'You are not authorized to unregister clusters from project [{}]. '
          'Underlying error: {}'.format(project, e))
    except apitools_exceptions.HttpNotFoundError:
      log.status.Print(
          'Membership [{}] for the cluster was not found on the fleet. '
          'It may already have been deleted, or it may never have existed.'
          .format(name))



# -*- coding: utf-8 -*- #
# Copyright 2019 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""'logging views list' command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.logging import util
from googlecloudsdk.calliope import base

DETAILED_HELP = {
    'DESCRIPTION': """
        Lists the available locations for Cloud Logging.
    """,
    'EXAMPLES': """
     To list the available locations, run:

        $ {command}
    """,
}


class List(base.ListCommand):
  """List the availables location."""

  @staticmethod
  def Args(parser):
    """Register flags for this command."""

    util.AddParentArgs(parser, 'locations to list')
    parser.display_info.AddFormat('table(locationId)')

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
      command invocation.

    Yields:
      The list of locations.
    """
    result = util.GetClient().projects_locations.List(
        util.GetMessages().LoggingProjectsLocationsListRequest(
            name=util.GetParentFromArgs(args)))

    for location in result.locations:
      yield location

List.detailed_help = DETAILED_HELP



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Update GitHub trigger command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.api_lib.cloudbuild import trigger_config as trigger_utils
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.cloudbuild import resource_args
from googlecloudsdk.command_lib.util.concepts import concept_parsers
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


class UpdateGitHub(base.UpdateCommand):
  """Update GitHub trigger used by Google Cloud Build."""

  detailed_help = {
      'EXAMPLES':
          """\
            To update the branch pattern of the trigger:

              $ {command} my-trigger --branch-pattern=".*"

            To update the build config of the trigger:

              $ {command} my-trigger --build-config="cloudbuild.yaml"

            To update the substitutions of the trigger:

              $ {command} my-trigger --update-substitutions=_REPO_NAME=my-repo,_BRANCH_NAME=master
        """,
  }

  @staticmethod
  def Args(parser):
    """Registers flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    concept_parsers.ConceptParser.ForResource(
        'TRIGGER',
        resource_args.GetTriggerResourceSpec(),
        'Build Trigger.',
        required=True).AddToParser(parser)

    flag_config = trigger_utils.AddTriggerArgs(
        parser, add_region_flag=False, add_name=False)
    gen_config = flag_config.add_mutually_exclusive_group()
    gen_config.add_argument(
        '--repository',
        help=("""\
Repository resource (2nd gen) to use, in the format
"projects/*/locations/*/connections/*/repositories/*".
"""),
    )
    v1_config = gen_config.add_argument_group()
    v1_config.add_argument(
        '--repo-owner', help='Owner of the GitHub Repository.')
    v1_config.add_argument(
        '--repo-name', help='Name of the GitHub Repository.')
    v1_config.add_argument(
        '--enterprise-config',
        help="""\
Resource name of the GitHub Enterprise config that should be applied to this
installation.

For example: "projects/{$project_id}/locations/{$location_id}/githubEnterpriseConfigs/{$config_id}
        """)
    ref_config = flag_config.add_mutually_exclusive_group()
    trigger_utils.AddBranchPattern(ref_config)
    trigger_utils.AddTagPattern(ref_config)
    pr_config = ref_config.add_argument_group(help='Pull Request settings')
    pr_config.add_argument(
        '--pull-request-pattern',
        metavar='REGEX',
        help="""\
A regular expression specifying which base git branch to match for
pull request events.

This pattern is used as a regex search for the base branch (the branch you are
trying to merge into) for pull request updates.
For example, --pull-request-pattern=foo will match "foo", "foobar", and "barfoo".

The syntax of the regular expressions accepted is the syntax accepted by
RE2 and described at https://github.com/google/re2/wiki/Syntax.
""")
    pr_config.add_argument(
        '--comment-control',
        choices={
            'COMMENTS_DISABLED': (
                'Do not require comments on Pull Requests before builds are'
                ' triggered.'
            ),
            'COMMENTS_ENABLED': (
                'Enforce that repository owners or collaborators must comment'
                ' on Pull Requests before builds are triggered.'
            ),
            'COMMENTS_ENABLED_FOR_EXTERNAL_CONTRIBUTORS_ONLY': (
                'Enforce that repository owners or collaborators must comment'
                " on external contributors' Pull Requests before builds are"
                ' triggered.'
            ),
        },
        default='COMMENTS_ENABLED',
        help=(
            "Require a repository collaborator or owner to comment '/gcbrun' on"
            ' a pull request before running the build.'
        ),
    )

    trigger_utils.AddBuildConfigArgsForUpdate(
        flag_config, has_build_config=True)
    trigger_utils.AddRepoEventArgs(flag_config)
    trigger_utils.AddIncludeLogsArgs(flag_config)

  def ParseTriggerFromFlags(self, args, old_trigger):
    """Parses command line arguments into a build trigger.

    Args:
      args: An argparse arguments object.
      old_trigger: The existing trigger to be updated.

    Returns:
      A build trigger object.

    Raises:
      RequiredArgumentException: If comment_control is defined but
      pull_request_pattern isn't.
    """
    messages = cloudbuild_util.GetMessagesModule()

    trigger, done = trigger_utils.ParseTriggerArgsForUpdate(args, messages)
    if done:
      return trigger

    # GitHub config
    gh = messages.GitHubEventsConfig(
        owner=args.repo_owner,
        name=args.repo_name,
        enterpriseConfigResourceName=args.enterprise_config)
    if args.pull_request_pattern:
      gh.pullRequest = messages.PullRequestFilter(
          branch=args.pull_request_pattern)
      if args.comment_control:
        gh.pullRequest.commentControl = (
            messages.PullRequestFilter.CommentControlValueValuesEnum(
                args.comment_control
            )
        )
    else:
      # Push event
      gh.push = messages.PushFilter(
          branch=args.branch_pattern, tag=args.tag_pattern)
    trigger.github = gh

    trigger_utils.ParseBuildConfigArgsForUpdate(
        trigger, old_trigger, args, messages, has_build_config=True)
    trigger_utils.ParseRepoEventArgs(trigger, args)
    trigger_utils.ParseIncludeLogsWithStatus(trigger, args, messages)

    return trigger

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: An argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      The updated GitHub trigger.
    """

    client = cloudbuild_util.GetClientInstance()
    messages = cloudbuild_util.GetMessagesModule()
    project = properties.VALUES.core.project.Get(required=True)
    regionprop = properties.VALUES.builds.region.Get()
    location = args.region or regionprop or cloudbuild_util.DEFAULT_REGION
    triggerid = args.TRIGGER

    name = resources.REGISTRY.Parse(
        triggerid,
        params={
            'projectsId': project,
            'locationsId': location,
            'triggersId': triggerid,
        },
        collection='cloudbuild.projects.locations.triggers').RelativeName()

    old_trigger = client.projects_locations_triggers.Get(
        client.MESSAGES_MODULE.CloudbuildProjectsLocationsTriggersGetRequest(
            name=name, triggerId=triggerid))

    trigger = self.ParseTriggerFromFlags(args, old_trigger)

    # Overwrite the substitutions.additionalProperties in updateMask.
    sub = 'substitutions'
    update_mask = cloudbuild_util.MessageToFieldPaths(trigger)
    update_mask = list(
        set(map(lambda m: sub if m.startswith(sub) else m, update_mask)))

    req = messages.CloudbuildProjectsLocationsTriggersPatchRequest(
        resourceName=name,
        triggerId=triggerid,
        buildTrigger=trigger,
        updateMask=','.join(update_mask))

    updated_trigger = client.projects_locations_triggers.Patch(req)
    log.UpdatedResource(triggerid, kind='build_trigger')

    return updated_trigger



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Gcloud firebase test argument files supplementary help."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.calliope import base


class TestingArgFiles(base.TopicCommand):
  """Supplementary help for arg-files to be used with *gcloud firebase test*."""

  # pylint: disable=line-too-long
  detailed_help = {
      'DESCRIPTION':
          """\
          {description}

          All *gcloud firebase test android run* arguments may be specified by
          flags on the command line and/or via a YAML-formatted _ARG_FILE_. The
          optional, positional ARG_SPEC argument on the command line is used to
          specify a single _ARG_FILE_:_ARG_GROUP_NAME_ pair, where _ARG_FILE_ is
          the path to the YAML argument file, and _ARG_GROUP_NAME_ is the name
          of the argument group to load and parse. The _ARG_FILE_ must contain
          valid YAML syntax or gcloud will respond with an error.

          The basic format of a YAML argument file is:

            arg-group1:
              arg1: value1  # a comment
              arg2: value2
              ...

            # Another comment
            arg-group2:
              arg3: value3
              ...

          List arguments may be specified within square brackets:

            directories-to-pull: [/sdcard/dir1, /data/dir2]

          or by using the alternate YAML list notation with one dash per list
          item:

            ```
            directories-to-pull:
              - /sdcard/dir1
              - /data/dir2
            ```

          If a list argument only contains a single value, you may omit the
          square brackets:

            directories-to-pull: /sdcard/dir1

          Composition

          A special *include: [_ARG_GROUP1_, ...]* syntax allows merging or
          composition of argument groups (see *EXAMPLES* below). Included
          argument groups can *include:* other argument groups within the
          same YAML file, with unlimited nesting.

          Precedence

          An argument which appears on the command line has the highest
          precedence and will override the same argument if it is specified
          within an argument file.

          Any argument defined directly within a group will have higher
          precedence than an identical argument which is merged into that
          group using the *include:* keyword.

          """,
      'EXAMPLES':
          """\

          Here are the contents of a very simple YAML argument file which
          is assumed to be stored in a file named excelsior_args.yaml:

            # Run a quick 'robo' test on the 'Excelsior' app for
            # 90 seconds using only the default Test Lab device.
            quick-robo-test:
              app: path/to/excelsior.apk
              type: robo
              max-steps: 100
              timeout: 90s
              async: true

          To invoke this test, run:

            $ gcloud firebase test android run excelsior_args.yaml:quick-robo-test

          To select which device(s) you wish to test against in an argument
          file, use *device:* to specify one or more devices, with each device
          having one or more dimensions. For example, to specify the LG G3
          device in the Chinese locale and with landscape orientation, use:

            single-device-group:
              device: [{model: g3, orientation: landscape, locale: zh}]

          To specify multiple devices, use any of the following equivalent YAML
          formats:

            multi-device-group1:
              device: [{model: flo}, {model: g3, version: 19, locale: zh}, {model: mako, version: 21}]

            multi-device-group2:
              device:
                - {model: flo}
                - {model: g3, version: 19, locale: zh}
                - {model: mako, version: 21}

            multi-device-group3:
              device:
                - model: flo
                - model: g3
                  version: 19
                  locale: zh
                - model: mako
                  version: 21

          If your app has a login screen, or has additional UI elements which
          require input text, you may specify the resource names of the Android
          target UI elements, along with their corresponding input values, in
          the 'robo-directives' map argument. You may also specify the elements
          which the Robo test should prioritize clicking. In the example below,
          "username_resource" is the resource name of the username field and
          "username" is the input for that field (similarly for password), and
          "signin_button_resource" is the resource name of the sign in button.

            # Run a 'robo' test on the 'Excelsior' app with login credentials.
            robo-test-with-login:
              app: path/to/excelsior.apk
              type: robo
              robo-directives:
                "text:username_resource": username
                "text:password_resource": password
                "click:sigin_button_resource": ""

          Assuming the above YAML text is appended to the arg-file named
          excelsior_args.yaml, you may invoke the test by running:

            $ gcloud firebase test android run excelsior_args.yaml:robo-test-with-login

          Here is a slightly more complicated example which demonstrates
          composition of argument groups using the legacy device dimension
          arguments (*device:* is now the preferred way to specify test
          devices). Assume the following YAML text is appended to the arg-file
          shown above named excelsior_args.yaml:

            # Specify some unit tests to be run against a test matrix
            # with one device type, two Android versions, and four
            # locales, for a total of eight test variations (1*2*4).
            unit-tests:
              type: instrumentation
              app: path/to/excelsior.apk
              test: path/to/excelsior-test.apk  # the unit tests
              timeout: 10m
              device-ids: NexusLowRes
              include: [supported-versions, supported-locales]

            supported-versions:
              os-version-ids: [21, 22]

            supported-locales:
              locales: [en, es, fr, it]

          To invoke this test matrix, run:

            $ gcloud firebase test android run excelsior_args.yaml:unit-tests

          To run these unit tests with the same locales and os-version-ids,
          but substituting a sampling of three physical Android devices
          instead of the single virtual NexusLowRes device, run:

            $ gcloud firebase test android run excelsior_args.yaml:unit-tests --device-ids shamu,htc_m8,g3

          In the last example, the --device-ids argument on the
          command line overrides the device-ids: specification inside the
          arg-file because command-line arguments have higher precedence.
          """,
  }



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fetch the IAM policy for a key."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.cloudkms import iam
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.kms import flags
from googlecloudsdk.command_lib.kms import resource_args


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.GA)
class GetIamPolicy(base.ListCommand):
  """Get the IAM policy for a key.

  Gets the IAM policy for the given key.

  Returns an empty policy if the resource does not have a policy
  set.

  ## EXAMPLES

  The following command gets the IAM policy for the key `frodo` within
  the keyring `fellowship` and location `global`:

    $ {command} frodo --keyring=fellowship --location=global
  """

  @staticmethod
  def Args(parser):
    resource_args.AddKmsKeyResourceArgForKMS(parser, True, 'key')
    base.URI_FLAG.RemoveFromParser(parser)

  def Run(self, args):
    return iam.GetCryptoKeyIamPolicy(flags.ParseCryptoKeyName(args))



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to simulate orgpolicy."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from apitools.base.py import encoding_helper
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.orgpolicy import utils as orgpolicy_utils
from googlecloudsdk.api_lib.policy_intelligence import orgpolicy_simulator
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.command_lib.policy_intelligence.simulator.orgpolicy import utils
from googlecloudsdk.core import log


_DETAILED_HELP_ALPHA = {
    'brief':
        """\
      Preview of Violations Service for OrgPolicy Simulator.
        """,
    'DESCRIPTION':
        """\
      Preview of Violations Service for OrgPolicy Simulator.
        """,
    'EXAMPLES':
        """\
      Simulate changes to Organization Policies:, run:

        $ {command}
        --organization=ORGANIZATION_ID
        --policy policy.json
        --custom-constraint custom-constraint.json

      See https://cloud.google.com/iam for more information about Org Policy Simulator.
      The official Org Policy Simulator document will be released soon.

      """
}


def _ArgsAlpha(parser):
  """Parses arguments for the commands."""
  parser.add_argument(
      '--organization',
      metavar='ORGANIZATION_ID',
      required=True,
      help=('Organization ID.'))

  parser.add_argument(
      '--policies',
      type=arg_parsers.ArgList(),
      metavar='POLICIES',
      action=arg_parsers.UpdateAction,
      help="""Path to the JSON or YAML file that contains the Org Policy to simulate.
      Multiple Policies can be simulated by providing multiple, comma-separated paths.
      E.g. --policies=p1.json,p2.json.
      The format of policy can be found and created by `gcloud org-policies set-policy`.
      See https://cloud.google.com/sdk/gcloud/reference/org-policies/set-policy for more details.
      """)

  parser.add_argument(
      '--custom-constraints',
      type=arg_parsers.ArgList(),
      metavar='CUSTOM_CONSTRAINTS',
      action=arg_parsers.UpdateAction,
      help="""Path to the JSON or YAML file that contains the Custom Constraints to simulate.
      Multiple Custom Constraints can be simulated by providing multiple, comma-separated paths.
      e.g., --custom-constraints=constraint1.json,constraint2.json.
      """)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
@base.Hidden
class SimulateAlpha(base.Command):
  """Simulate the Org Policies."""

  detailed_help = _DETAILED_HELP_ALPHA

  @staticmethod
  def Args(parser):
    """Parses arguments for the commands."""
    _ArgsAlpha(parser)

  def Run(self, args):
    if not args.policies and not args.custom_constraints:
      raise exceptions.ConflictingArgumentsException(
          'Must specify either --policies or --custom-constraints or both.')

    orgpolicy_simulator_api = orgpolicy_simulator.OrgPolicySimulatorApi(
        self.ReleaseTrack())
    # Parse files and get Policy Overlay
    policies = []
    if args.policies:
      for policy_file in args.policies:
        policy = utils.GetPolicyMessageFromFile(policy_file,
                                                self.ReleaseTrack())
        if not policy.name:
          raise exceptions.InvalidArgumentException(
              'Policy name',
              "'name' field not present in the organization policy.")
        policy_parent = orgpolicy_utils.GetResourceFromPolicyName(
            policy.name)
        policy_overlay = orgpolicy_simulator_api.GetOrgPolicyPolicyOverlay(
            policy=policy,
            policy_parent=policy_parent)
        policies.append(policy_overlay)

    # Parse files and get Custom Constraints Overlay
    custom_constraints = []
    if args.custom_constraints:
      for custom_constraint_file in args.custom_constraints:
        custom_constraint = utils.GetCustomConstraintMessageFromFile(
            custom_constraint_file,
            self.ReleaseTrack())
        if not custom_constraint.name:
          raise exceptions.InvalidArgumentException(
              'Custom constraint name',
              "'name' field not present in the custom constraint.")
        custom_constraint_parent = orgpolicy_utils.GetResourceFromPolicyName(
            custom_constraint.name)
        constraint_overlay = orgpolicy_simulator_api.GetOrgPolicyCustomConstraintOverlay(
            custom_constraint=custom_constraint,
            custom_constraint_parent=custom_constraint_parent)
        custom_constraints.append(constraint_overlay)

    overlay = orgpolicy_simulator_api.GetOrgPolicyOverlay(
        policies=policies, custom_constraints=custom_constraints)

    # Generate Violations Preview and get long operation id
    organization_resource = 'organizations/' + args.organization
    parent = utils.GetParentFromOrganization(organization_resource)
    violations = orgpolicy_simulator_api.GetPolicysimulatorOrgPolicyViolationsPreview(
        overlay=overlay)
    request = orgpolicy_simulator_api.GenerateOrgPolicyViolationsPreviewRequest(
        violations_preview=violations,
        parent=parent)
    op_simulator_service = orgpolicy_simulator_api.client.OrganizationsLocationsService(
        orgpolicy_simulator_api.client)
    violations_preview_operation = op_simulator_service.OrgPolicyViolationsPreviews(
        request=request)

    # Poll Long Running Operation and get Violations Preview
    operation_response_raw = orgpolicy_simulator_api.WaitForOperation(
        violations_preview_operation,
        'Waiting for operation [{}] to complete'.format(
            violations_preview_operation.name))

    violations_preview = encoding_helper.JsonToMessage(
        orgpolicy_simulator_api.messages
        .GoogleCloudPolicysimulatorV1alphaOrgPolicyViolationsPreview,
        encoding_helper.MessageToJson(operation_response_raw))

    if not violations_preview.violationsCount or not violations_preview.resourceCounts:
      log.err.Print('No violations found in the violations preview.\n')

    # List results of the Violations under Violations Preview.
    list_violations_request = orgpolicy_simulator_api.messages.PolicysimulatorOrganizationsLocationsOrgPolicyViolationsPreviewsOrgPolicyViolationsListRequest(
        parent=violations_preview.name)
    pov_service = orgpolicy_simulator_api.client.OrganizationsLocationsOrgPolicyViolationsPreviewsOrgPolicyViolationsService(
        orgpolicy_simulator_api.client)

    return list_pager.YieldFromList(
        pov_service,
        list_violations_request,
        batch_size=1000,
        field='orgPolicyViolations',
        batch_size_attribute='pageSize')




"""Generated client library for gameservices version v1beta."""
# NOTE: This file is autogenerated and should not be edited by hand.

from __future__ import absolute_import

from apitools.base.py import base_api
from googlecloudsdk.generated_clients.apis.gameservices.v1beta import gameservices_v1beta_messages as messages


class GameservicesV1beta(base_api.BaseApiClient):
  """Generated client library for service gameservices version v1beta."""

  MESSAGES_MODULE = messages
  BASE_URL = 'https://gameservices.googleapis.com/'
  MTLS_BASE_URL = 'https://gameservices.mtls.googleapis.com/'

  _PACKAGE = 'gameservices'
  _SCOPES = ['https://www.googleapis.com/auth/cloud-platform']
  _VERSION = 'v1beta'
  _CLIENT_ID = 'CLIENT_ID'
  _CLIENT_SECRET = 'CLIENT_SECRET'
  _USER_AGENT = 'google-cloud-sdk'
  _CLIENT_CLASS_NAME = 'GameservicesV1beta'
  _URL_VERSION = 'v1beta'
  _API_KEY = None

  def __init__(self, url='', credentials=None,
               get_credentials=True, http=None, model=None,
               log_request=False, log_response=False,
               credentials_args=None, default_global_params=None,
               additional_http_headers=None, response_encoding=None):
    """Create a new gameservices handle."""
    url = url or self.BASE_URL
    super(GameservicesV1beta, self).__init__(
        url, credentials=credentials,
        get_credentials=get_credentials, http=http, model=model,
        log_request=log_request, log_response=log_response,
        credentials_args=credentials_args,
        default_global_params=default_global_params,
        additional_http_headers=additional_http_headers,
        response_encoding=response_encoding)
    self.projects_locations_gameServerDeployments_configs = self.ProjectsLocationsGameServerDeploymentsConfigsService(self)
    self.projects_locations_gameServerDeployments = self.ProjectsLocationsGameServerDeploymentsService(self)
    self.projects_locations_operations = self.ProjectsLocationsOperationsService(self)
    self.projects_locations_realms_gameServerClusters = self.ProjectsLocationsRealmsGameServerClustersService(self)
    self.projects_locations_realms = self.ProjectsLocationsRealmsService(self)
    self.projects_locations = self.ProjectsLocationsService(self)
    self.projects = self.ProjectsService(self)

  class ProjectsLocationsGameServerDeploymentsConfigsService(base_api.BaseApiService):
    """Service class for the projects_locations_gameServerDeployments_configs resource."""

    _NAME = 'projects_locations_gameServerDeployments_configs'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsGameServerDeploymentsConfigsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new game server config in a given project, location, and game server deployment. Game server configs are immutable, and are not applied until referenced in the game server deployment rollout resource.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsConfigsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/configs',
        http_method='POST',
        method_id='gameservices.projects.locations.gameServerDeployments.configs.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['configId'],
        relative_path='v1beta/{+parent}/configs',
        request_field='gameServerConfig',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsConfigsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single game server config. The deletion fails if the game server config is referenced in a game server deployment rollout.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsConfigsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/configs/{configsId}',
        http_method='DELETE',
        method_id='gameservices.projects.locations.gameServerDeployments.configs.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsConfigsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single game server config.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsConfigsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GameServerConfig) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/configs/{configsId}',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.configs.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsConfigsGetRequest',
        response_type_name='GameServerConfig',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists game server configs in a given project, location, and game server deployment.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsConfigsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGameServerConfigsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/configs',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.configs.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/configs',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsConfigsListRequest',
        response_type_name='ListGameServerConfigsResponse',
        supports_download=False,
    )

  class ProjectsLocationsGameServerDeploymentsService(base_api.BaseApiService):
    """Service class for the projects_locations_gameServerDeployments resource."""

    _NAME = 'projects_locations_gameServerDeployments'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsGameServerDeploymentsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new game server deployment in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments',
        http_method='POST',
        method_id='gameservices.projects.locations.gameServerDeployments.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['deploymentId'],
        relative_path='v1beta/{+parent}/gameServerDeployments',
        request_field='gameServerDeployment',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single game server deployment.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}',
        http_method='DELETE',
        method_id='gameservices.projects.locations.gameServerDeployments.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def FetchDeploymentState(self, request, global_params=None):
      r"""Retrieves information about the current state of the game server deployment. Gathers all the Agones fleets and Agones autoscalers, including fleets running an older version of the game server deployment.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsFetchDeploymentStateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (FetchDeploymentStateResponse) The response message.
      """
      config = self.GetMethodConfig('FetchDeploymentState')
      return self._RunMethod(
          config, request, global_params=global_params)

    FetchDeploymentState.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}:fetchDeploymentState',
        http_method='POST',
        method_id='gameservices.projects.locations.gameServerDeployments.fetchDeploymentState',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}:fetchDeploymentState',
        request_field='fetchDeploymentStateRequest',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsFetchDeploymentStateRequest',
        response_type_name='FetchDeploymentStateResponse',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single game server deployment.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GameServerDeployment) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsGetRequest',
        response_type_name='GameServerDeployment',
        supports_download=False,
    )

    def GetIamPolicy(self, request, global_params=None):
      r"""Gets the access control policy for a resource. Returns an empty policy if the resource exists and does not have a policy set.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsGetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('GetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}:getIamPolicy',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.getIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=['options_requestedPolicyVersion'],
        relative_path='v1beta/{+resource}:getIamPolicy',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsGetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def GetRollout(self, request, global_params=None):
      r"""Gets details of a single game server deployment rollout.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsGetRolloutRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GameServerDeploymentRollout) The response message.
      """
      config = self.GetMethodConfig('GetRollout')
      return self._RunMethod(
          config, request, global_params=global_params)

    GetRollout.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/rollout',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.getRollout',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}/rollout',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsGetRolloutRequest',
        response_type_name='GameServerDeploymentRollout',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists game server deployments in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGameServerDeploymentsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments',
        http_method='GET',
        method_id='gameservices.projects.locations.gameServerDeployments.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/gameServerDeployments',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsListRequest',
        response_type_name='ListGameServerDeploymentsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Patches a game server deployment.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}',
        http_method='PATCH',
        method_id='gameservices.projects.locations.gameServerDeployments.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta/{+name}',
        request_field='gameServerDeployment',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PreviewRollout(self, request, global_params=None):
      r"""Previews the game server deployment rollout. This API does not mutate the rollout resource.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsPreviewRolloutRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PreviewGameServerDeploymentRolloutResponse) The response message.
      """
      config = self.GetMethodConfig('PreviewRollout')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreviewRollout.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/rollout:preview',
        http_method='PATCH',
        method_id='gameservices.projects.locations.gameServerDeployments.previewRollout',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['previewTime', 'updateMask'],
        relative_path='v1beta/{+name}/rollout:preview',
        request_field='gameServerDeploymentRollout',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsPreviewRolloutRequest',
        response_type_name='PreviewGameServerDeploymentRolloutResponse',
        supports_download=False,
    )

    def SetIamPolicy(self, request, global_params=None):
      r"""Sets the access control policy on the specified resource. Replaces any existing policy. Can return `NOT_FOUND`, `INVALID_ARGUMENT`, and `PERMISSION_DENIED` errors.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsSetIamPolicyRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Policy) The response message.
      """
      config = self.GetMethodConfig('SetIamPolicy')
      return self._RunMethod(
          config, request, global_params=global_params)

    SetIamPolicy.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}:setIamPolicy',
        http_method='POST',
        method_id='gameservices.projects.locations.gameServerDeployments.setIamPolicy',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:setIamPolicy',
        request_field='setIamPolicyRequest',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsSetIamPolicyRequest',
        response_type_name='Policy',
        supports_download=False,
    )

    def TestIamPermissions(self, request, global_params=None):
      r"""Returns permissions that a caller has on the specified resource. If the resource does not exist, this will return an empty set of permissions, not a `NOT_FOUND` error. Note: This operation is designed to be used for building permission-aware UIs and command-line tools, not for authorization checking. This operation may "fail open" without warning.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsTestIamPermissionsRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (TestIamPermissionsResponse) The response message.
      """
      config = self.GetMethodConfig('TestIamPermissions')
      return self._RunMethod(
          config, request, global_params=global_params)

    TestIamPermissions.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}:testIamPermissions',
        http_method='POST',
        method_id='gameservices.projects.locations.gameServerDeployments.testIamPermissions',
        ordered_params=['resource'],
        path_params=['resource'],
        query_params=[],
        relative_path='v1beta/{+resource}:testIamPermissions',
        request_field='testIamPermissionsRequest',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsTestIamPermissionsRequest',
        response_type_name='TestIamPermissionsResponse',
        supports_download=False,
    )

    def UpdateRollout(self, request, global_params=None):
      r"""Patches a single game server deployment rollout. The method will not return an error if the update does not affect any existing realms. For example, the following cases will not return an error: * The default_game_server_config is changed but all existing realms use the override. * A non-existing realm is explicitly called out in the game_server_config_overrides field.

      Args:
        request: (GameservicesProjectsLocationsGameServerDeploymentsUpdateRolloutRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('UpdateRollout')
      return self._RunMethod(
          config, request, global_params=global_params)

    UpdateRollout.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/gameServerDeployments/{gameServerDeploymentsId}/rollout',
        http_method='PATCH',
        method_id='gameservices.projects.locations.gameServerDeployments.updateRollout',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta/{+name}/rollout',
        request_field='gameServerDeploymentRollout',
        request_type_name='GameservicesProjectsLocationsGameServerDeploymentsUpdateRolloutRequest',
        response_type_name='Operation',
        supports_download=False,
    )

  class ProjectsLocationsOperationsService(base_api.BaseApiService):
    """Service class for the projects_locations_operations resource."""

    _NAME = 'projects_locations_operations'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsOperationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Cancel(self, request, global_params=None):
      r"""Starts asynchronous cancellation on a long-running operation. The server makes a best effort to cancel the operation, but success is not guaranteed. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`. Clients can use Operations.GetOperation or other methods to check whether the cancellation succeeded or whether the operation completed despite cancellation. On successful cancellation, the operation is not deleted; instead, it becomes an operation with an Operation.error value with a google.rpc.Status.code of 1, corresponding to `Code.CANCELLED`.

      Args:
        request: (GameservicesProjectsLocationsOperationsCancelRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Cancel')
      return self._RunMethod(
          config, request, global_params=global_params)

    Cancel.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}:cancel',
        http_method='POST',
        method_id='gameservices.projects.locations.operations.cancel',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}:cancel',
        request_field='cancelOperationRequest',
        request_type_name='GameservicesProjectsLocationsOperationsCancelRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a long-running operation. This method indicates that the client is no longer interested in the operation result. It does not cancel the operation. If the server doesn't support this method, it returns `google.rpc.Code.UNIMPLEMENTED`.

      Args:
        request: (GameservicesProjectsLocationsOperationsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Empty) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='DELETE',
        method_id='gameservices.projects.locations.operations.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsOperationsDeleteRequest',
        response_type_name='Empty',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets the latest state of a long-running operation. Clients can use this method to poll the operation result at intervals as recommended by the API service.

      Args:
        request: (GameservicesProjectsLocationsOperationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations/{operationsId}',
        http_method='GET',
        method_id='gameservices.projects.locations.operations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsOperationsGetRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists operations that match the specified filter in the request. If the server doesn't support this method, it returns `UNIMPLEMENTED`. NOTE: the `name` binding allows API services to override the binding to use different resource name schemes, such as `users/*/operations`. To override the binding, API services can add a binding such as `"/v1/{name=users/*}/operations"` to their service configuration. For backwards compatibility, the default name includes the operations collection id, however overriding users must ensure the name binding is the parent resource, without the operations collection id.

      Args:
        request: (GameservicesProjectsLocationsOperationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListOperationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/operations',
        http_method='GET',
        method_id='gameservices.projects.locations.operations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/operations',
        request_field='',
        request_type_name='GameservicesProjectsLocationsOperationsListRequest',
        response_type_name='ListOperationsResponse',
        supports_download=False,
    )

  class ProjectsLocationsRealmsGameServerClustersService(base_api.BaseApiService):
    """Service class for the projects_locations_realms_gameServerClusters resource."""

    _NAME = 'projects_locations_realms_gameServerClusters'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsRealmsGameServerClustersService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new game server cluster in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters',
        http_method='POST',
        method_id='gameservices.projects.locations.realms.gameServerClusters.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['gameServerClusterId'],
        relative_path='v1beta/{+parent}/gameServerClusters',
        request_field='gameServerCluster',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single game server cluster.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters/{gameServerClustersId}',
        http_method='DELETE',
        method_id='gameservices.projects.locations.realms.gameServerClusters.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single game server cluster.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (GameServerCluster) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters/{gameServerClustersId}',
        http_method='GET',
        method_id='gameservices.projects.locations.realms.gameServerClusters.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['view'],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersGetRequest',
        response_type_name='GameServerCluster',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists game server clusters in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListGameServerClustersResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters',
        http_method='GET',
        method_id='gameservices.projects.locations.realms.gameServerClusters.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken', 'view'],
        relative_path='v1beta/{+parent}/gameServerClusters',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersListRequest',
        response_type_name='ListGameServerClustersResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Patches a single game server cluster.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters/{gameServerClustersId}',
        http_method='PATCH',
        method_id='gameservices.projects.locations.realms.gameServerClusters.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta/{+name}',
        request_field='gameServerCluster',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PreviewCreate(self, request, global_params=None):
      r"""Previews creation of a new game server cluster in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersPreviewCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PreviewCreateGameServerClusterResponse) The response message.
      """
      config = self.GetMethodConfig('PreviewCreate')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreviewCreate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters:previewCreate',
        http_method='POST',
        method_id='gameservices.projects.locations.realms.gameServerClusters.previewCreate',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['gameServerClusterId', 'previewTime', 'view'],
        relative_path='v1beta/{+parent}/gameServerClusters:previewCreate',
        request_field='gameServerCluster',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersPreviewCreateRequest',
        response_type_name='PreviewCreateGameServerClusterResponse',
        supports_download=False,
    )

    def PreviewDelete(self, request, global_params=None):
      r"""Previews deletion of a single game server cluster.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersPreviewDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PreviewDeleteGameServerClusterResponse) The response message.
      """
      config = self.GetMethodConfig('PreviewDelete')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreviewDelete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters/{gameServerClustersId}:previewDelete',
        http_method='DELETE',
        method_id='gameservices.projects.locations.realms.gameServerClusters.previewDelete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['previewTime'],
        relative_path='v1beta/{+name}:previewDelete',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersPreviewDeleteRequest',
        response_type_name='PreviewDeleteGameServerClusterResponse',
        supports_download=False,
    )

    def PreviewUpdate(self, request, global_params=None):
      r"""Previews updating a GameServerCluster.

      Args:
        request: (GameservicesProjectsLocationsRealmsGameServerClustersPreviewUpdateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PreviewUpdateGameServerClusterResponse) The response message.
      """
      config = self.GetMethodConfig('PreviewUpdate')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreviewUpdate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}/gameServerClusters/{gameServerClustersId}:previewUpdate',
        http_method='PATCH',
        method_id='gameservices.projects.locations.realms.gameServerClusters.previewUpdate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['previewTime', 'updateMask'],
        relative_path='v1beta/{+name}:previewUpdate',
        request_field='gameServerCluster',
        request_type_name='GameservicesProjectsLocationsRealmsGameServerClustersPreviewUpdateRequest',
        response_type_name='PreviewUpdateGameServerClusterResponse',
        supports_download=False,
    )

  class ProjectsLocationsRealmsService(base_api.BaseApiService):
    """Service class for the projects_locations_realms resource."""

    _NAME = 'projects_locations_realms'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsRealmsService, self).__init__(client)
      self._upload_configs = {
          }

    def Create(self, request, global_params=None):
      r"""Creates a new realm in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsRealmsCreateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Create')
      return self._RunMethod(
          config, request, global_params=global_params)

    Create.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms',
        http_method='POST',
        method_id='gameservices.projects.locations.realms.create',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['realmId'],
        relative_path='v1beta/{+parent}/realms',
        request_field='realm',
        request_type_name='GameservicesProjectsLocationsRealmsCreateRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Delete(self, request, global_params=None):
      r"""Deletes a single realm.

      Args:
        request: (GameservicesProjectsLocationsRealmsDeleteRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Delete')
      return self._RunMethod(
          config, request, global_params=global_params)

    Delete.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}',
        http_method='DELETE',
        method_id='gameservices.projects.locations.realms.delete',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsDeleteRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def Get(self, request, global_params=None):
      r"""Gets details of a single realm.

      Args:
        request: (GameservicesProjectsLocationsRealmsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Realm) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}',
        http_method='GET',
        method_id='gameservices.projects.locations.realms.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsGetRequest',
        response_type_name='Realm',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists realms in a given project and location.

      Args:
        request: (GameservicesProjectsLocationsRealmsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListRealmsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms',
        http_method='GET',
        method_id='gameservices.projects.locations.realms.list',
        ordered_params=['parent'],
        path_params=['parent'],
        query_params=['filter', 'orderBy', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+parent}/realms',
        request_field='',
        request_type_name='GameservicesProjectsLocationsRealmsListRequest',
        response_type_name='ListRealmsResponse',
        supports_download=False,
    )

    def Patch(self, request, global_params=None):
      r"""Patches a single realm.

      Args:
        request: (GameservicesProjectsLocationsRealmsPatchRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Operation) The response message.
      """
      config = self.GetMethodConfig('Patch')
      return self._RunMethod(
          config, request, global_params=global_params)

    Patch.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}',
        http_method='PATCH',
        method_id='gameservices.projects.locations.realms.patch',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['updateMask'],
        relative_path='v1beta/{+name}',
        request_field='realm',
        request_type_name='GameservicesProjectsLocationsRealmsPatchRequest',
        response_type_name='Operation',
        supports_download=False,
    )

    def PreviewUpdate(self, request, global_params=None):
      r"""Previews patches to a single realm.

      Args:
        request: (GameservicesProjectsLocationsRealmsPreviewUpdateRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (PreviewRealmUpdateResponse) The response message.
      """
      config = self.GetMethodConfig('PreviewUpdate')
      return self._RunMethod(
          config, request, global_params=global_params)

    PreviewUpdate.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}/realms/{realmsId}:previewUpdate',
        http_method='PATCH',
        method_id='gameservices.projects.locations.realms.previewUpdate',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['previewTime', 'updateMask'],
        relative_path='v1beta/{+name}:previewUpdate',
        request_field='realm',
        request_type_name='GameservicesProjectsLocationsRealmsPreviewUpdateRequest',
        response_type_name='PreviewRealmUpdateResponse',
        supports_download=False,
    )

  class ProjectsLocationsService(base_api.BaseApiService):
    """Service class for the projects_locations resource."""

    _NAME = 'projects_locations'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsLocationsService, self).__init__(client)
      self._upload_configs = {
          }

    def Get(self, request, global_params=None):
      r"""Gets information about a location.

      Args:
        request: (GameservicesProjectsLocationsGetRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (Location) The response message.
      """
      config = self.GetMethodConfig('Get')
      return self._RunMethod(
          config, request, global_params=global_params)

    Get.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations/{locationsId}',
        http_method='GET',
        method_id='gameservices.projects.locations.get',
        ordered_params=['name'],
        path_params=['name'],
        query_params=[],
        relative_path='v1beta/{+name}',
        request_field='',
        request_type_name='GameservicesProjectsLocationsGetRequest',
        response_type_name='Location',
        supports_download=False,
    )

    def List(self, request, global_params=None):
      r"""Lists information about the supported locations for this service.

      Args:
        request: (GameservicesProjectsLocationsListRequest) input message
        global_params: (StandardQueryParameters, default: None) global arguments
      Returns:
        (ListLocationsResponse) The response message.
      """
      config = self.GetMethodConfig('List')
      return self._RunMethod(
          config, request, global_params=global_params)

    List.method_config = lambda: base_api.ApiMethodInfo(
        flat_path='v1beta/projects/{projectsId}/locations',
        http_method='GET',
        method_id='gameservices.projects.locations.list',
        ordered_params=['name'],
        path_params=['name'],
        query_params=['filter', 'includeUnrevealedLocations', 'pageSize', 'pageToken'],
        relative_path='v1beta/{+name}/locations',
        request_field='',
        request_type_name='GameservicesProjectsLocationsListRequest',
        response_type_name='ListLocationsResponse',
        supports_download=False,
    )

  class ProjectsService(base_api.BaseApiService):
    """Service class for the projects resource."""

    _NAME = 'projects'

    def __init__(self, client):
      super(GameservicesV1beta.ProjectsService, self).__init__(client)
      self._upload_configs = {
          }



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Job-specific printer."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

from frozendict import frozendict
from googlecloudsdk.command_lib.run.integrations import deployment_states
from googlecloudsdk.command_lib.run.integrations.formatters import cloudsql_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import domain_routing_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import fallback_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import redis_formatter
from googlecloudsdk.command_lib.run.integrations.formatters import states
from googlecloudsdk.core.console import console_attr
from googlecloudsdk.core.resource import custom_printer_base as cp

INTEGRATION_PRINTER_FORMAT = 'integration'

_FALLBACK_FORMATTER = fallback_formatter.FallbackFormatter()
_INTEGRATION_FORMATTER_MAPS = frozendict({
    'custom-domains': domain_routing_formatter.DomainRoutingFormatter(),
    'redis': redis_formatter.RedisFormatter(),
    'cloudsql': cloudsql_formatter.CloudSQLFormatter(),
})


class Record(object):
  """Record holds data that is passed around to printers for formatting.

  Attributes:
    name: str, name of the integration
    region: str, GCP region for the integration.
    integration_type: str, type of the integration, for example: redis,
      custom-domains, or cloudsql.
    config: dict, resource config for the given integration.
    status: dict, application status for the given integration.
    latest_deployment:
      str, canonical deployment name for the latest deployment for the given
      integration.
  """

  def __init__(self, name, region, integration_type, config, status,
               latest_deployment):
    self.name = name
    self.region = region
    self.integration_type = integration_type
    self.config = config if config is not None else {}
    self.status = status if status is not None else {}
    self.latest_deployment = latest_deployment


class IntegrationPrinter(cp.CustomPrinterBase):
  """Prints the run Integration in a custom human-readable format."""

  def Transform(self, record):
    """Transform an integration into the output structure of marker classes."""
    formatter = GetFormatter(record.integration_type)
    config_block = formatter.TransformConfig(record)
    component_block = (
        formatter.TransformComponentStatus(record)
        if record.status
        else 'Status not available')

    lines = [
        self.Header(record),
        ' ',
        self._DeploymentProgress(record.latest_deployment,
                                 formatter),
        config_block,
        ' ',
        cp.Labeled([
            cp.Lines([
                'Integration Components',
                component_block
            ])
        ]),
    ]

    call_to_action = formatter.CallToAction(record)
    if call_to_action:
      lines.append(' ')
      lines.append(call_to_action)

    return cp.Lines(lines)

  def Header(self, record):
    """Print the header of the integration.

    Args:
      record: dict, the integration.

    Returns:
      The printed output.
    """
    con = console_attr.GetConsoleAttr()
    formatter = GetFormatter(record.integration_type)
    resource_state = record.status.get('state', states.UNKNOWN)
    symbol = formatter.StatusSymbolAndColor(resource_state)
    return con.Emphasize('{} Integration status: {} in region {}'.format(
        symbol, record.name, record.region))

  def _DeploymentProgress(self, deployment, formatter):
    """Returns a message denoting the deployment progress.

    If there is no ongoing deployment and the deployment was successful, then
    this will be empty.

    Currently this only shows something if the latest deployment was a failure.
    In the future this will be updated to show more granular statuses as the
    deployment is ongoing.

    Args:
      deployment:  The deployment object
      formatter: The specific formatter used for the integration type.

    Returns:
      str, The message denoting the most recent deployment's progress (failure).
    """
    if deployment is None:
      return ''

    state = str(deployment.status.state)

    if state == deployment_states.FAILED:
      reason = deployment.status.errorMessage
      symbol = formatter.StatusSymbolAndColor(states.FAILED)
      return '{} Latest deployment: FAILED - {}\n'.format(symbol, reason)

    return ''


def GetFormatter(integration_type):
  """Returns the formatter for the given integration type.

  Args:
    integration_type: string, the integration type.

  Returns:
    A formatter object.
  """
  return _INTEGRATION_FORMATTER_MAPS.get(integration_type,
                                         _FALLBACK_FORMATTER)



# -*- coding: utf-8 -*- #
# Copyright 2021 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""`gcloud dataplex content create` command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.dataplex import content
from googlecloudsdk.api_lib.dataplex import util as dataplex_util
from googlecloudsdk.api_lib.util import exceptions as gcloud_exception
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.dataplex import resource_args
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.args import labels_util
from googlecloudsdk.core import log


@base.ReleaseTracks(base.ReleaseTrack.ALPHA, base.ReleaseTrack.GA)
class Create(base.Command):
  """Creating a content."""

  detailed_help = {
      'EXAMPLES':
          """\

          To create a Dataplex content `test-content` of type notebook  within lake `test-lake` in location `us-central1`.

           $ {command} --project=test-project --location=us-central1 --lake=test-lake --kernel-type=PYTHON3 --data-text='' --path='test-content'

          """,
  }

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: argparse.Parser: Parser object for command line inputs.
    """
    # Content resource does not take a Content ID and auto-generates it.
    # Hence we add Lake Resource Args here and not Content Resource args.
    resource_args.AddLakeResourceArg(
        parser, 'to create a Content to.', positional=False)
    parser.add_argument('--description', help='Description of the Content')
    parser.add_argument(
        '--data-text', help='Content data in string format', required=True)
    parser.add_argument(
        '--path',
        help='The path for the Content file, represented as directory structure',
        required=True)

    sqlscript_or_notebook = parser.add_group(
        required=True,
        mutex=True,
        help='Sql script or notebook related configurations.')

    notebook = sqlscript_or_notebook.add_group(
        required=False, help='Notebook related configurations.')

    notebook.add_argument(
        '--kernel-type',
        choices={'PYTHON3': 'python3'},
        type=arg_utils.ChoiceToEnumName,
        help='Kernel Type of the notebook.',
        required=True)

    sql_script = sqlscript_or_notebook.add_group(
        required=False, help='Sql script related configurations.')

    sql_script.add_argument(
        '--query-engine',
        choices={'SPARK': 'spark'},
        type=arg_utils.ChoiceToEnumName,
        help='Query Engine to be used for the Sql Query.',
        required=True)

    parser.add_argument(
        '--validate-only',
        action='store_true',
        default=False,
        help='Validate the create action, but don\'t actually perform it.')
    labels_util.AddCreateLabelsFlags(parser)

  @gcloud_exception.CatchHTTPErrorRaiseHTTPException(
      'Status code: {status_code}. {status_message}.')
  def Run(self, args):
    """Constructs and sends request.

    Args:
      args: argparse.Namespace, An object that contains the values for the
        arguments specified in the .Args() method.
    """
    # We extract lake reference, instead of content reference as content id
    # is not provided at the time of create content and is auto-generated
    # and returned in response.
    lake_ref = args.CONCEPTS.lake.Parse()
    dataplex_client = dataplex_util.GetClientInstance()
    content_response = dataplex_client.projects_locations_lakes_contentitems.Create(
        dataplex_util.GetMessageModule(
        ).DataplexProjectsLocationsLakesContentitemsCreateRequest(
            parent=lake_ref.RelativeName(),
            validateOnly=args.validate_only,
            googleCloudDataplexV1Content=content
            .GenerateContentForCreateRequest(args)))

    validate_only = getattr(args, 'validate_only', False)
    if validate_only:
      log.status.Print('Validation complete.')
      return

    log.CreatedResource(
        content_response.name,
        details='Content created in lake [{0}] in project [{1}] with location [{2}]'
        .format(lake_ref.lakesId, lake_ref.projectsId, lake_ref.locationsId))



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""services list command."""

from __future__ import absolute_import
from __future__ import division
from __future__ import unicode_literals

from googlecloudsdk.api_lib.services import services_util
from googlecloudsdk.api_lib.services import serviceusage
from googlecloudsdk.calliope import base


class List(base.ListCommand):
  """List services for a project.

  This command lists the services that are enabled or available to be enabled
  by a project. You can choose the mode in which the command will list
  services by using exactly one of the `--enabled` or `--available` flags.
  `--enabled` is the default.

  ## EXAMPLES

  To list the services the current project has enabled for consumption, run:

    $ {command} --enabled

  To list the services the current project can enable for consumption, run:

    $ {command} --available
  """

  @staticmethod
  def Args(parser):
    """Args is called by calliope to gather arguments for this command.

    Args:
      parser: An argparse parser that you can use to add arguments that go
          on the command line after this command. Positional arguments are
          allowed.
    """
    mode_group = parser.add_mutually_exclusive_group(required=False)

    mode_group.add_argument(
        '--enabled',
        action='store_true',
        help=('(DEFAULT) Return the services which the '
              'project has enabled.'))

    mode_group.add_argument(
        '--available',
        action='store_true',
        help=('Return the services available to the '
              'project to enable. This list will '
              'include any services that the project '
              'has already enabled.'))

    base.PAGE_SIZE_FLAG.SetDefault(parser, 200)

    # Remove unneeded list-related flags from parser
    base.URI_FLAG.RemoveFromParser(parser)

    parser.display_info.AddFormat("""
          table(
            config.name:label=NAME:sort=1,
            config.title
          )
        """)

  def Run(self, args):
    """Run 'services list'.

    Args:
      args: argparse.Namespace, The arguments that this command was invoked
          with.

    Returns:
      The list of services for this project.
    """
    # Default mode is --enabled, so if no flags were specified,
    # turn on the args.enabled flag.
    if not (args.enabled or args.available):
      args.enabled = True
    project = services_util.GetValidatedProject(args.project)
    return serviceusage.ListServices(project, args.enabled, args.page_size,
                                     args.limit)
